unit Unit1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls;

type

  { TForm1 }

  TForm1 = class(TForm)
    Button1: TButton;
    Label1: TLabel;
    Memo1: TMemo;
    procedure Button1Click(Sender: TObject);
  private
    { private declarations }
  public
    { public declarations }
  end;

var
  Form1: TForm1; //Global variable
  x_global: integer;
  GlobalIntegerResult: integer = 0;  //Global variable initialized to 0
  GlobalIntegerInput: integer = 1; //Global variable initialized to 1


implementation

{$R *.lfm}

{ TForm1 }

{procedures examples
Notice that Local & Global vars
can be accessed by procedures and functions}


procedure PrintHelloWorld();
begin
   Form1.Memo1.Lines.Add('Hello World');
end;

procedure PrintReceipt(ProductName:string; Price:integer);
begin
   Form1.Memo1.Lines.Add('Product : ' + ProductName +
                         ', Price : ' + IntToStr(Price));
end;

//procedure pass by value example:
procedure Increase(x:integer);
begin
   x:=x+1; //x is now 2+1=3
   Form1.Memo1.Lines.Add('Local variable x=' + IntToStr(x)); //prints 3
end;

//Result sent in a Form1.Label.Caption (global variable)
procedure DoubleInLabel(x:integer);
var LocalResult:integer; //Local variable
begin
   LocalResult:=2*x;
   Form1.Label1.Caption := IntToStr(LocalResult);
end;

//Result passed as Global Variable [not recommended]
procedure DoubleInGlobalIntegerResult(x:integer);
var LocalResult:integer; //Local variable
begin
   LocalResult:=2*x;
   GlobalIntegerResult := 2*x;
end;

//Input and Result passed as Global Variables [not recommended]
procedure DoubleUsingGlobals();
var LocalResult:integer; //Local variable
begin
   LocalResult:=2*GlobalIntegerInput;
   GlobalIntegerResult := LocalResult;
end;

{Overloaded routines must be declared  with the overload directive

 OVERLOAD IN LAZARUS

 Prior to version 1.9 of the compiler,the overloaded functions needed to be in the same unit.
 Now the compiler will continue searching in other units if it doesn’t find a matching version
 of an overloaded function in one unit, and if the overload keyword is present.

 If the overload keyword is not present,then all overloaded versions must reside in the
 same unit,and if it concerns methods part of a class, they must be in the same class,
 i.e. the compiler will not look for overloaded methods in parent classes if the overload
 keyword was not specified.

 ***GENERALLY OVERLOAD IS USED ON THE SAME CLASS***

 }


procedure TripleInMemo(x:integer); overload; //I can be overloaded ...
begin
   Form1.Memo1.Lines.Add('Three times the integer ' + IntToStr(x) + '=' + IntToStr(3*x));
end;

procedure TripleInMemo(x:double); overload; //I can be overloaded ...
begin
   Form1.Memo1.Lines.Add('Three times the float ' + FloatToStr(x) + '=' +  FloatToStr(3*x));
end;

procedure TripleInMemo(x:string); overload; //I can be overloaded ...
begin
   Form1.Memo1.Lines.Add('Three times the string : ' + x + x + x);
end;

procedure TripleInMemo(x,y:double); overload; //I can be overloaded ...
begin
   Form1.Memo1.Lines.Add('Three times the float ' + FloatToStr(x) + '=' +  FloatToStr(3*x));
   Form1.Memo1.Lines.Add('Three times the float ' + FloatToStr(y) + '=' +  FloatToStr(3*y));
end;

/////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////
//functions examples

function Sum(a,b:integer):integer; //returns an integer
begin
   result:=a+b;
end;

function DoubleIt(x:integer):integer; overload; //result
var LocalVariable : integer;
begin
   LocalVariable:=2*x;
   result:=LocalVariable;
end;

function DoubleIt(x:double):double; overload;//result
var LocalVariable : double;
begin
   LocalVariable:=2*x;
   result:=LocalVariable;
end;

function DoubleIt(x:string):string; overload;//result
begin
   result:=x+x;
end;



//declaration order example :

{
//wrong !!!
function CapitalizeAndDoubleString(s:string):string;
var str:string;
begin
   str:=CapitalizeString(s); //wrong it is not declared yet
   result := str+str;
end;
}

//Capitalisation is the writing of a word with its first letter
//in uppercase and the remaining letters in lowercase.
function  CapitalizeString(s:string):string;
var str:string; //local var
begin
     if Length(s)>=1 then begin

        str := copy(s, 1, 1);
        str := UpperCase(str);

        if Length(s)>=2 then begin
           str := str + LowerCase(copy(s, 2, Length(s)));
        end;
        result:=str;

     end else begin //length = 0
         result:='';
     end;
end;

//correct !!!
function CapitalizeAndDoubleString(s:string):string;
var str:string;
begin
   str:=CapitalizeString(s); // declared before CapitalizeAndDoubleString
   result := str+str;
end;


/////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////
//recursion examples :

{Factorial definition :
!n = n*(n-1)*(n-2)*...*2*1
!0 = 1 (by definition)
example : 5!=5*4*3*2*1=120
}
function Factorial(n:integer):Int64;
var res : int64;
    i : integer;
begin
   if n>1 then begin
     res:=1;
     //n! = n*(n-1)*(n-2)*...*2 (*1 is not needed !)
     for i:=n downto 2 do begin
       res := res * i;
     end;
     result:=res;
   end else begin // !1=1 and !0=1
     result:=1;
   end;
end;

function FactorialRecursion(n:integer):Int64;
begin
   if n>1 then begin
      //n! = n*(n-1)*(n-2)*...*2*1 = n*(n-1)!
      result:=n * FactorialRecursion(n-1);
   end else begin // 1!=1 and 0!=1
     result:=1;
   end;
end;

{triangle number
Sigma(n)=n+(n-1)+(n-2)+...+2+1
Sigma(n=0)=0 (by definition)
Example : Sigma(5)=5+4+3+2+1=12
}

function TriangleNumber(n:integer):Int64;
var res : int64;
    i : integer;
begin
   if n>0 then begin
     res:=0;
     //Sigma(n)=n+(n-1)+(n-2)+...+2+1
     for i:=n downto 1 do begin
       res := res + i;
     end;
     result:=res;
   end else begin // Sigma(0)=0
     result:=0;
   end;
end;

function TriangleNumberRecursion(n:integer):Int64;
begin
   if n>0 then begin
      //Sigma(n)=n+(n-1)+(n-2)+...+2+1 = n + Sigma(n-1)
      result:=n + TriangleNumberRecursion(n-1);
   end else begin  // Sigma(0)=0
     result:=0;
   end;
end;





//////////////////////////////////////////////////////////
//EVENTS
//////////////////////////////////////////////////////////

procedure TForm1.Button1Click(Sender: TObject);
var  x : integer; //local variable
begin

   PrintHelloWorld();
   PrintReceipt('DesktopZoomer', 15);

   {Scopes
   Local Scope : As you can see the LocalResult local variable
   which is located internally at the DoubleInLabel Procedure
   dies after the end of the procedure.}
   //Memo1.Lines.Add(IntToStr(LocalResult));  // Error: Identifier not found "LocalResult"
   {Global Scope :
   Instead : }
   //Memo1.Lines.Add(IntToStr(GlobalIntegerResult)); //is correct
   {can be accesses since it is a global variable}

   //procedure pass by value examples:
   Memo1.Lines.Add('Procedure pass by value local');
   x:=2; //initialize local var at 2
   Memo1.Lines.Add('Initial value x =' + IntToStr(x)); //prints 2
   Increase(x); //this procedure localy changes x to 3 and prints 3
   //after the Increase procedure execution x (local variable) remains = 2
   Memo1.Lines.Add('after the increase procedure the local value of x =' + IntToStr(x));

   Memo1.Lines.Add('--------------------------------');

   Memo1.Lines.Add('Procedure pass by value global');
   x_global:=5; //initialize global var at 5
   Memo1.Lines.Add('Initial value x =' + IntToStr(x_global)); //prints 5
   Increase(x_global); //this procedure locally changes x to 6 and prints 6
   //after the Increase procedure execution x (global variable) remains = 5
   Memo1.Lines.Add('after the increase procedure the global value of x =' + IntToStr(x_global));

   Memo1.Lines.Add('--------------------------------');



   //procedure that outputs in a Label.Caption (Global)
   DoubleInLabel(5);

   //procedure that outputs in GlobalIntegerResult
   DoubleInGlobalIntegerResult(5); //5*2=10
   //Now print the GlobalIntegerResult
   Memo1.Lines.Add('Print GlobalIntegerResult=' + IntToStr(GlobalIntegerResult));

   //procedure that uses global input and output
   //Do not use. Demonstration only :
   GlobalIntegerInput:=100;
   DoubleUsingGlobals(); //100*2=200
   Memo1.Lines.Add('Print GlobalIntegerResult=' + IntToStr(GlobalIntegerResult));


   //overload procedure example:
   TripleInMemo(4);
   TripleInMemo(4.2);
   TripleInMemo('Simple string.');
   TripleInMemo(5.1, 2.4);


   //function examples :
   Memo1.Lines.Add('function Sum 8+2=' + IntToStr(Sum(8,2)));

   //function procedure example:
   Memo1.Lines.Add('function 5x2=' + IntToStr(DoubleIt(5)));
   Memo1.Lines.Add('function 5.2x2=' + FloatToStr(DoubleIt(5.2)));
   Memo1.Lines.Add('function double string:' + DoubleIt('Test string.'));

   //function call another function
   //ordering functions
   Memo1.Lines.Add('Capitalize & double string:' + CapitalizeAndDoubleString('hello world.'));

   //recursion
   Memo1.Lines.Add('Simple Function : 8!=' + IntToStr(Factorial(8)));
   Memo1.Lines.Add('Recursive Function : 8! =' + IntToStr(FactorialRecursion(8)));
   Memo1.Lines.Add('Simple Function : TriangleNumber(10)=' + IntToStr(TriangleNumber(10)));
   Memo1.Lines.Add('Recursive Function : TriangleNumber(10)=' + IntToStr(TriangleNumber(10)));
end;

end.

